#include "SimpleLocTracker.hpp"
#include <SFML/System/Vector2.hpp>


void SimpleLocTracker::start(sf::Vector2f to_pos, float to_rot, int total_encodings) {
    last_pos = get_pos();
    last_rot = get_rot();
    current_mission = Mission {
      .from_pos = last_pos, .to_pos = to_pos,
      .from_rot = last_rot, .to_rot = to_rot,
      .total_encodings = total_encodings, .current_encodings = 0,
    };
}

void SimpleLocTracker::progress() {
    if (!current_mission.has_value()) return;
    current_mission.value().current_encodings += 1;
}

inline float interpolate(float from, float to, float fac) {
    auto inv = 1. - fac;
    return from * inv + to * fac;
}

inline sf::Vector2f interpolate(sf::Vector2f from, sf::Vector2f to, float fac) {
    return sf::Vector2f(
        interpolate(from.x, to.x, fac),
        interpolate(from.y, to.y, fac)
    );
}

sf::Vector2f SimpleLocTracker::get_pos() {
    if (!current_mission.has_value()) return last_pos;
    return interpolate(current_mission.value().from_pos, current_mission.value().to_pos, current_mission.value().progress());
}

float SimpleLocTracker::get_rot() {
    if (!current_mission.has_value()) return last_rot;
    return interpolate(current_mission.value().from_rot, current_mission.value().to_rot, current_mission.value().progress());
}
